/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#ifndef fesslix_octave_H
#define fesslix_octave_H

#include "flxlibs_interface.h"



class FlxOctaveBox {
  private:
    #if FLX_DEBUG
      /**
      * @brief number of instances of this calss - only one class may exist
      */
      static int Cinst;
    #endif
    bool initialized;    
    
    void init_octave_actual();
  public:
    FlxOctaveBox();
    ~FlxOctaveBox();
    
    void init_octave() { if (initialized) return; else init_octave_actual(); }
};


// -------------------------- OBJECTS ----------------------------------------------------------------------


/**
* @brief object class: 
*/
class FlxObjOctaveMtx : public FlxObjBase {
  private:
    const std::string funname;
    FlxString* resVecName;
    FlxMtxConstFun* inpVecName;
    
    void task();
  public:
    FlxObjOctaveMtx ( const bool dolog, const std::string& funname, FlxString* resVecName, FlxMtxConstFun* inpVecName ) : FlxObjBase(dolog), funname(funname), resVecName(resVecName), inpVecName(inpVecName) {}
    ~FlxObjOctaveMtx() { delete resVecName; delete inpVecName;  }
};

/**
* @brief object read class: for FlxObjOctaveMtx
*/
class FlxObjReadOctaveMtx : public FlxObjReadBase {
  public:
    FlxObjReadOctaveMtx() : FlxObjReadBase(true) {}
    FlxObjBase* read ();
};

// --------------------------------------------------------------------------------------

/**
* @brief object class: define a Matrix-constant
*
* octave_addpath("PATH");
*/
class FlxOctave_addpath : public FlxObjBase {
  private:
    FlxString* Path;
    void task();
  public:
    FlxOctave_addpath ( const bool dolog, FlxString* Path );
    ~FlxOctave_addpath() { delete Path; }
};

/**
* @brief object read class: for FlxOctave_addpath
*/
class FlxObjReadOctave_addpath : public FlxObjReadBase {
  public:
    FlxObjReadOctave_addpath() : FlxObjReadBase(true) {}
    FlxObjBase* read ();
};



// -------------------------- FUNCTIONS ----------------------------------------------------------------------


/**
* @brief arithmetic class: represents a LSF defined in Octave
*/
class FunOctaveLSF : public FunBaseFun_multPara, public FlxDataBase {
  private:
    const std::string funname;
    RBRV_constructor* RndBox;
    const tuint NOX;
    flxVec rvv;
    std::string rbrvsetn;
  public:
    FunOctaveLSF (const std::string& funname, RBRV_constructor* RndBox, std::string rbrvsetn, std::vector<FunBase*> *ParaListV);
    ~FunOctaveLSF();
    const tdouble calc();
    const bool search_circref(FlxFunction* fcr) { return false; }
    const std::string write_v();
    const std::string write();
    const bool dependOn_Const(const tdouble* const thenumber) { return false; }
    const bool optimize(FunBasePtr& optf, const Fun_OptimizeInfo &foi) { return false; }
};

class FunReadFunOctaveLSF : public FunReadFunBase, public FlxDataBase {
  public:
    FunReadFunOctaveLSF() : FunReadFunBase(true) {}
    FunBase* read ( bool errSerious );
};

// --------------------------------------------------------------------------------------

/**
* @brief arithmetic class: represents a LSF defined in Octave
*/
class FunOctaveFun : public FunBaseFun_multPara, public FlxDataBase {
  private:
    const std::string funname;
    FlxMtxConstFun* inpVecF;
    FlxObjBase* loopblock;
  public:
    FunOctaveFun (const std::string& funname, FlxMtxConstFun* inpVecF, FlxObjBase* loopblock, std::vector<FunBase*> *ParaListV);
    ~FunOctaveFun();
    const tdouble calc();
    const bool search_circref(FlxFunction* fcr) { return false; }
    const std::string write_v();
    const std::string write();
    const bool dependOn_Const(const tdouble* const thenumber);
    const bool optimize(FunBasePtr& optf, const Fun_OptimizeInfo &foi) { return false; }
};

class FunReadFunOctaveFun : public FunReadFunBase {
  public:
    FunReadFunOctaveFun() : FunReadFunBase(true) {}
    FunBase* read ( bool errSerious );
};

// --------------------------------------------------------------------------------------

/**
* @brief object class: executes a command 'cmd' in Octave
*
* matlab_cmd("CMD");
*/
class FlxOctave_cmd : public FlxObjBase {
  private:
    FlxString* cmd;
    void task();
  public:
    FlxOctave_cmd ( const bool dolog, FlxString* cmd ) : FlxObjBase(dolog), cmd(cmd) {}
    ~FlxOctave_cmd() { delete cmd; }
};

/**
* @brief object read class: for FlxOctave_cmd
*/
class FlxObjReadOctave_cmd : public FlxObjReadBase {
  public:
    FlxObjReadOctave_cmd() : FlxObjReadBase(true) {}
    FlxObjBase* read ();
};

// --------------------------------------------------------------------------------------

/**
* @brief object class: sets a variable in Octave
*
* octave_setvar scalar: scalarname=FlxFunction;
* octave_setvar vector: vecname=FlxMtxFun;
*/
class FlxOctave_setvar : public FlxObjBase {
  private:
    FlxString* vname;
    FlxFunction* sfun;
    FlxMtxConstFun* mfun;
    void task();
  public:
    FlxOctave_setvar ( const bool dolog, FlxString* vname, FlxFunction* sfun, FlxMtxConstFun* mfun ) : FlxObjBase(dolog), vname(vname), sfun(sfun), mfun(mfun) {}
    ~FlxOctave_setvar();
};

/**
* @brief object read class: for FlxOctave_setvar
*/
class FlxObjReadOctave_setvar : public FlxObjReadBase {
  public:
    FlxObjReadOctave_setvar() : FlxObjReadBase(true) {}
    FlxObjBase* read ();
};


// --------------------------------------------------------------------------------------

/**
* @brief arithmetic class: return the scalar value of an Octave variable
*/
class FunOctaveGetVar : public FunBaseFun, public FlxDataBase {
  private:
    const std::string varname;
  public:
    FunOctaveGetVar (const std::string& varname) : varname(varname) {}
    const tdouble calc();
    const bool search_circref(FlxFunction* fcr) { return false; }
    const std::string write_v();
    const std::string write();
    const bool dependOn_Const(const tdouble* const thenumber) { return false; }
    const bool optimize(FunBasePtr& optf, const Fun_OptimizeInfo &foi) { return false; }
};

class FunReadFunOctaveGetVar : public FunReadFunBase, public FlxDataBase {
  public:
    FunReadFunOctaveGetVar() : FunReadFunBase(true) {}
    FunBase* read ( bool errSerious );
};

// --------------------------------------------------------------------------------------



#endif // fesslix_octave_H

